@@ -104,9 +104,21 @@ extern int32_t thiszone;	/* seconds offset from gmt to local time */
  * that "snapend - (l)" underflows.
  *
  * The check is for <= rather than < because "l" might be 0.
+ *
+ * We cast the pointers to uintptr_t to make sure that the compiler
+ * doesn't optimize away any of these tests (which it is allowed to
+ * do, as adding an integer to, or subtracting an integer from, a
+ * pointer assumes that the pointer is a pointer to an element of an
+ * array and that the result of the addition or subtraction yields a
+ * pointer to another member of the array, so that, for example, if
+ * you subtract a positive integer from a pointer, the result is
+ * guaranteed to be less than the original pointer value). See
+ *
+ *	http://www.kb.cert.org/vuls/id/162289
  */
-#define TTEST2(var, l) (snapend - (l) <= snapend && \
-			(const u_char *)&(var) <= snapend - (l))
+#define TTEST2(var, l) \
+	((uintptr_t)snapend - (l) <= (uintptr_t)snapend && \
+	   (uintptr_t)&(var) <= (uintptr_t)snapend - (l))
 
 /* True if "var" was captured */
 #define TTEST(var) TTEST2(var, sizeof(var))
@@ -148,7 +160,6 @@ extern uint16_t create_osi_cksum(const uint8_t *, int, int);
 
 #include <pcap.h>
 
-extern char *q922_string(const u_char *);
 extern char *smb_errstr(int, int);
 extern const char *nt_errstr(uint32_t);
 
