@@ -41,11 +41,6 @@
 #include <config.h>
 #endif
 
-#ifndef lint
-static const char rcsid[] _U_ =
-     "@(#) $Header: /tcpdump/master/tcpdump/missing/getnameinfo.c,v 1.11 2003-11-16 09:36:49 guy Exp $";
-#endif
-
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <net/if.h>
@@ -112,7 +107,7 @@ getnameinfo(sa, salen, host, hostlen, serv, servlen, flags)
 	u_short port;
 	int family, i;
 	char *addr, *p;
-	u_int32_t v4a;
+	uint32_t v4a;
 	int h_error;
 	char numserv[512];
 	char numaddr[512];
@@ -168,7 +163,7 @@ getnameinfo(sa, salen, host, hostlen, serv, servlen, flags)
 
 	switch (sa->sa_family) {
 	case AF_INET:
-		v4a = (u_int32_t)
+                v4a = (uint32_t)
 			ntohl(((struct sockaddr_in *)sa)->sin_addr.s_addr);
 		if (IN_MULTICAST(v4a) || IN_EXPERIMENTAL(v4a))
 			flags |= NI_NUMERICHOST;
