@@ -36,13 +36,6 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: inet_aton.c,v 1.6 2003-11-16 09:36:49 guy Exp $ */
-
-#ifndef lint
-static const char rcsid[] _U_ =
-     "@(#) $Header: /tcpdump/master/tcpdump/missing/inet_aton.c,v 1.6 2003-11-16 09:36:49 guy Exp $";
-#endif
-
 #include <tcpdump-stdinc.h>
 
 /* Minimal implementation of inet_aton.
