@@ -31,17 +31,10 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: snprintf.c,v 1.8 2003-11-16 09:36:51 guy Exp $ */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#ifndef lint
-static const char rcsid[] _U_ =
-     "@(#) $Header: /tcpdump/master/tcpdump/missing/snprintf.c,v 1.8 2003-11-16 09:36:51 guy Exp $";
-#endif
-
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
