@@ -40,11 +40,7 @@
  * Western Research Laboratory
  */
 
-#ifndef lint
-static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/parsenfsfh.c,v 1.29 2006-06-13 22:21:38 guy Exp $ (LBL)";
-#endif
-
+#define NETDISSECT_REWORKED
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -83,7 +79,7 @@ static const char rcsid[] _U_ =
 
 #ifdef	ultrix
 /* Nasty hack to keep the Ultrix C compiler from emitting bogus warnings */
-#define	XFF(x)	((u_int32_t)(x))
+#define	XFF(x)	((uint32_t)(x))
 #else
 #define	XFF(x)	(x)
 #endif
@@ -110,17 +106,14 @@ static const char rcsid[] _U_ =
 static int is_UCX(const unsigned char *);
 
 void
-Parse_fh(fh, len, fsidp, inop, osnamep, fsnamep, ourself)
-register const unsigned char *fh;
-int len _U_;
-my_fsid *fsidp;
-u_int32_t *inop;
-const char **osnamep;		/* if non-NULL, return OS name here */
-const char **fsnamep;		/* if non-NULL, return server fs name here (for VMS) */
-int ourself;		/* true if file handle was generated on this host */
+Parse_fh(register const unsigned char *fh, int len _U_, my_fsid *fsidp,
+	 uint32_t *inop,
+	 const char **osnamep, /* if non-NULL, return OS name here */
+	 const char **fsnamep, /* if non-NULL, return server fs name here (for VMS) */
+	 int ourself)	/* true if file handle was generated on this host */
 {
 	register const unsigned char *fhp = fh;
-	u_int32_t temp;
+	uint32_t temp;
 	int fhtype = FHT_UNKNOWN;
 	int i;
 
@@ -371,7 +364,7 @@ int ourself;		/* true if file handle was generated on this host */
 		memcpy((char *)fsidp, (char *)fh, 14);
 	    }
 	    else {
-		u_int32_t tempa[4];	/* at least 16 bytes, maybe more */
+		uint32_t tempa[4];	/* at least 16 bytes, maybe more */
 
 		memset((char *)tempa, 0, sizeof(tempa));
 		memcpy((char *)tempa, (char *)fh, 14); /* ensure alignment */
@@ -450,14 +443,13 @@ int ourself;		/* true if file handle was generated on this host */
  *	(3) followed by string of nulls
  */
 static int
-is_UCX(fhp)
-const unsigned char *fhp;
+is_UCX(const unsigned char *fhp)
 {
 	register int i;
 	int seen_null = 0;
 
 	for (i = 1; i < 14; i++) {
-	    if (isprint(fhp[i])) {
+	    if (ND_ISPRINT(fhp[i])) {
 		if (seen_null)
 		   return(0);
 		else
