@@ -36,27 +36,21 @@
  * RFC1701 (GRE), RFC1702 (GRE IPv4), and RFC2637 (Enhanced GRE)
  */
 
-#ifndef lint
-static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-gre.c,v 1.28 2005-04-06 21:32:39 mcr Exp $ (LBL)";
-#endif
-
+#define NETDISSECT_REWORKED
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include <tcpdump-stdinc.h>
 
-#include <stdio.h>
 #include <string.h>
 
 #include "interface.h"
-#include "addrtoname.h"
 #include "extract.h"
-
-#include "ip.h"
 #include "ethertype.h"
 
+static const char tstr[] = "[|gre]";
+
 #define	GRE_CP		0x8000		/* checksum present */
 #define	GRE_RP		0x4000		/* routing present */
 #define	GRE_KP		0x2000		/* key present */
@@ -67,9 +61,9 @@ static const char rcsid[] _U_ =
 
 static const struct tok gre_flag_values[] = {
     { GRE_CP, "checksum present"},
-    { GRE_RP, "routing present"}, 
-    { GRE_KP, "key present"}, 
-    { GRE_SP, "sequence# present"}, 
+    { GRE_RP, "routing present"},
+    { GRE_KP, "key present"},
+    { GRE_SP, "sequence# present"},
     { GRE_sP, "source routing present"},
     { GRE_RECRS, "recursion count"},
     { GRE_AP, "ack present"},
@@ -82,49 +76,47 @@ static const struct tok gre_flag_values[] = {
 #define	GRESRE_IP	0x0800		/* IP */
 #define	GRESRE_ASN	0xfffe		/* ASN */
 
-void gre_print_0(const u_char *, u_int);
-void gre_print_1(const u_char *, u_int);
-void gre_sre_print(u_int16_t, u_int8_t, u_int8_t, const u_char *, u_int);
-void gre_sre_ip_print(u_int8_t, u_int8_t, const u_char *, u_int);
-void gre_sre_asn_print(u_int8_t, u_int8_t, const u_char *, u_int);
+static void gre_print_0(netdissect_options *, const u_char *, u_int);
+static void gre_print_1(netdissect_options *, const u_char *, u_int);
+static void gre_sre_print(netdissect_options *, uint16_t, uint8_t, uint8_t, const u_char *, u_int);
+static void gre_sre_ip_print(netdissect_options *, uint8_t, uint8_t, const u_char *, u_int);
+static void gre_sre_asn_print(netdissect_options *, uint8_t, uint8_t, const u_char *, u_int);
 
 void
-gre_print(const u_char *bp, u_int length)
+gre_print(netdissect_options *ndo, const u_char *bp, u_int length)
 {
 	u_int len = length, vers;
 
 	if (len < 2) {
-		printf("[|gre]");
+		ND_PRINT((ndo, "%s", tstr));
 		return;
 	}
 	vers = EXTRACT_16BITS(bp) & GRE_VERS_MASK;
-        printf("GREv%u",vers);
+        ND_PRINT((ndo, "GREv%u",vers));
 
         switch(vers) {
         case 0:
-            gre_print_0(bp, len);
+            gre_print_0(ndo, bp, len);
             break;
         case 1:
-            gre_print_1(bp, len);
+            gre_print_1(ndo, bp, len);
             break;
 	default:
-            printf(" ERROR: unknown-version");
+            ND_PRINT((ndo, " ERROR: unknown-version"));
             break;
         }
-	return;
-
 }
 
-void
-gre_print_0(const u_char *bp, u_int length)
+static void
+gre_print_0(netdissect_options *ndo, const u_char *bp, u_int length)
 {
 	u_int len = length;
-	u_int16_t flags, prot;
+	uint16_t flags, prot;
 
 	flags = EXTRACT_16BITS(bp);
-        if (vflag)
-            printf(", Flags [%s]",
-                   bittok2str(gre_flag_values,"none",flags));
+        if (ndo->ndo_vflag)
+            ND_PRINT((ndo, ", Flags [%s]",
+                   bittok2str(gre_flag_values,"none",flags)));
 
 	len -= 2;
 	bp += 2;
@@ -138,14 +130,14 @@ gre_print_0(const u_char *bp, u_int length)
 	if ((flags & GRE_CP) | (flags & GRE_RP)) {
 		if (len < 2)
 			goto trunc;
-		if (vflag)
-			printf(", sum 0x%x", EXTRACT_16BITS(bp));
+		if (ndo->ndo_vflag)
+			ND_PRINT((ndo, ", sum 0x%x", EXTRACT_16BITS(bp)));
 		bp += 2;
 		len -= 2;
 
 		if (len < 2)
 			goto trunc;
-		printf(", off 0x%x", EXTRACT_16BITS(bp));
+		ND_PRINT((ndo, ", off 0x%x", EXTRACT_16BITS(bp)));
 		bp += 2;
 		len -= 2;
 	}
@@ -153,7 +145,7 @@ gre_print_0(const u_char *bp, u_int length)
 	if (flags & GRE_KP) {
 		if (len < 4)
 			goto trunc;
-		printf(", key=0x%x", EXTRACT_32BITS(bp));
+		ND_PRINT((ndo, ", key=0x%x", EXTRACT_32BITS(bp)));
 		bp += 4;
 		len -= 4;
 	}
@@ -161,16 +153,16 @@ gre_print_0(const u_char *bp, u_int length)
 	if (flags & GRE_SP) {
 		if (len < 4)
 			goto trunc;
-		printf(", seq %u", EXTRACT_32BITS(bp));
+		ND_PRINT((ndo, ", seq %u", EXTRACT_32BITS(bp)));
 		bp += 4;
 		len -= 4;
 	}
 
 	if (flags & GRE_RP) {
 		for (;;) {
-			u_int16_t af;
-			u_int8_t sreoff;
-			u_int8_t srelen;
+			uint16_t af;
+			uint8_t sreoff;
+			uint8_t srelen;
 
 			if (len < 4)
 				goto trunc;
@@ -183,7 +175,7 @@ gre_print_0(const u_char *bp, u_int length)
 			if (af == 0 && srelen == 0)
 				break;
 
-			gre_sre_print(af, sreoff, srelen, bp, len);
+			gre_sre_print(ndo, af, sreoff, srelen, bp, len);
 
 			if (len < srelen)
 				goto trunc;
@@ -192,64 +184,62 @@ gre_print_0(const u_char *bp, u_int length)
 		}
 	}
 
-        if (eflag)
-            printf(", proto %s (0x%04x)",
+        if (ndo->ndo_eflag)
+            ND_PRINT((ndo, ", proto %s (0x%04x)",
                    tok2str(ethertype_values,"unknown",prot),
-                   prot);
+                   prot));
 
-        printf(", length %u",length);
+        ND_PRINT((ndo, ", length %u",length));
 
-        if (vflag < 1)
-            printf(": "); /* put in a colon as protocol demarc */
+        if (ndo->ndo_vflag < 1)
+            ND_PRINT((ndo, ": ")); /* put in a colon as protocol demarc */
         else
-            printf("\n\t"); /* if verbose go multiline */
+            ND_PRINT((ndo, "\n\t")); /* if verbose go multiline */
 
 	switch (prot) {
 	case ETHERTYPE_IP:
-	        ip_print(gndo, bp, len);
+	        ip_print(ndo, bp, len);
 		break;
-#ifdef INET6
 	case ETHERTYPE_IPV6:
-		ip6_print(gndo, bp, len);
+		ip6_print(ndo, bp, len);
 		break;
-#endif
 	case ETHERTYPE_MPLS:
-		mpls_print(bp, len);
+		mpls_print(ndo, bp, len);
 		break;
 	case ETHERTYPE_IPX:
-		ipx_print(bp, len);
+		ipx_print(ndo, bp, len);
 		break;
 	case ETHERTYPE_ATALK:
-		atalk_print(bp, len);
+		atalk_print(ndo, bp, len);
 		break;
 	case ETHERTYPE_GRE_ISO:
-		isoclns_print(bp, len, len);
+		isoclns_print(ndo, bp, len, len);
 		break;
 	case ETHERTYPE_TEB:
-		ether_print(gndo, bp, len, len, NULL, NULL);
+		ether_print(ndo, bp, len, len, NULL, NULL);
 		break;
 	default:
-		printf("gre-proto-0x%x", prot);
+		ND_PRINT((ndo, "gre-proto-0x%x", prot));
 	}
 	return;
 
 trunc:
-	printf("[|gre]");
+	ND_PRINT((ndo, "%s", tstr));
 }
 
-void
-gre_print_1(const u_char *bp, u_int length)
+static void
+gre_print_1(netdissect_options *ndo, const u_char *bp, u_int length)
 {
 	u_int len = length;
-	u_int16_t flags, prot;
+	uint16_t flags, prot;
 
 	flags = EXTRACT_16BITS(bp);
 	len -= 2;
 	bp += 2;
 
-	if (vflag)
-            printf(", Flags [%s]",
-                   bittok2str(gre_flag_values,"none",flags));
+	if (ndo->ndo_vflag)
+            ND_PRINT((ndo, ", Flags [%s]",
+                   bittok2str(gre_flag_values,"none",flags)));
 
 	if (len < 2)
 		goto trunc;
@@ -259,12 +249,12 @@ gre_print_1(const u_char *bp, u_int length)
 
 
 	if (flags & GRE_KP) {
-		u_int32_t k;
+		uint32_t k;
 
 		if (len < 4)
 			goto trunc;
 		k = EXTRACT_32BITS(bp);
-		printf(", call %d", k & 0xffff);
+		ND_PRINT((ndo, ", call %d", k & 0xffff));
 		len -= 4;
 		bp += 4;
 	}
@@ -272,7 +262,7 @@ gre_print_1(const u_char *bp, u_int length)
 	if (flags & GRE_SP) {
 		if (len < 4)
 			goto trunc;
-		printf(", seq %u", EXTRACT_32BITS(bp));
+		ND_PRINT((ndo, ", seq %u", EXTRACT_32BITS(bp)));
 		bp += 4;
 		len -= 4;
 	}
@@ -280,78 +270,80 @@ gre_print_1(const u_char *bp, u_int length)
 	if (flags & GRE_AP) {
 		if (len < 4)
 			goto trunc;
-		printf(", ack %u", EXTRACT_32BITS(bp));
+		ND_PRINT((ndo, ", ack %u", EXTRACT_32BITS(bp)));
 		bp += 4;
 		len -= 4;
 	}
 
 	if ((flags & GRE_SP) == 0)
-		printf(", no-payload");
+		ND_PRINT((ndo, ", no-payload"));
 
-        if (eflag)
-            printf(", proto %s (0x%04x)",
+        if (ndo->ndo_eflag)
+            ND_PRINT((ndo, ", proto %s (0x%04x)",
                    tok2str(ethertype_values,"unknown",prot),
-                   prot);
+                   prot));
 
-        printf(", length %u",length);
+        ND_PRINT((ndo, ", length %u",length));
 
         if ((flags & GRE_SP) == 0)
             return;
 
-        if (vflag < 1)
-            printf(": "); /* put in a colon as protocol demarc */
+        if (ndo->ndo_vflag < 1)
+            ND_PRINT((ndo, ": ")); /* put in a colon as protocol demarc */
         else
-            printf("\n\t"); /* if verbose go multiline */
+            ND_PRINT((ndo, "\n\t")); /* if verbose go multiline */
 
 	switch (prot) {
 	case ETHERTYPE_PPP:
-                ppp_print(bp, len);
+		ppp_print(ndo, bp, len);
 		break;
 	default:
-		printf("gre-proto-0x%x", prot);
+		ND_PRINT((ndo, "gre-proto-0x%x", prot));
 		break;
 	}
 	return;
 
 trunc:
-	printf("[|gre]");
+	ND_PRINT((ndo, "%s", tstr));
 }
 
-void
-gre_sre_print(u_int16_t af, u_int8_t sreoff, u_int8_t srelen,
-    const u_char *bp, u_int len)
+static void
+gre_sre_print(netdissect_options *ndo, uint16_t af, uint8_t sreoff,
+    uint8_t srelen, const u_char *bp, u_int len)
 {
 	switch (af) {
 	case GRESRE_IP:
-		printf(", (rtaf=ip");
-		gre_sre_ip_print(sreoff, srelen, bp, len);
-		printf(") ");
+		ND_PRINT((ndo, ", (rtaf=ip"));
+		gre_sre_ip_print(ndo, sreoff, srelen, bp, len);
+		ND_PRINT((ndo, ") "));
 		break;
 	case GRESRE_ASN:
-		printf(", (rtaf=asn");
-		gre_sre_asn_print(sreoff, srelen, bp, len);
-		printf(") ");
+		ND_PRINT((ndo, ", (rtaf=asn"));
+		gre_sre_asn_print(ndo, sreoff, srelen, bp, len);
+		ND_PRINT((ndo, ") "));
 		break;
 	default:
-		printf(", (rtaf=0x%x) ", af);
+		ND_PRINT((ndo, ", (rtaf=0x%x) ", af));
 	}
 }
-void
-gre_sre_ip_print(u_int8_t sreoff, u_int8_t srelen, const u_char *bp, u_int len)
+
+static void
+gre_sre_ip_print(netdissect_options *ndo, uint8_t sreoff, uint8_t srelen,
+                 const u_char *bp, u_int len)
 {
 	struct in_addr a;
 	const u_char *up = bp;
 
 	if (sreoff & 3) {
-		printf(", badoffset=%u", sreoff);
+		ND_PRINT((ndo, ", badoffset=%u", sreoff));
 		return;
 	}
 	if (srelen & 3) {
-		printf(", badlength=%u", srelen);
+		ND_PRINT((ndo, ", badlength=%u", srelen));
 		return;
 	}
 	if (sreoff >= srelen) {
-		printf(", badoff/len=%u/%u", sreoff, srelen);
+		ND_PRINT((ndo, ", badoff/len=%u/%u", sreoff, srelen));
 		return;
 	}
 
@@ -360,9 +352,9 @@ gre_sre_ip_print(u_int8_t sreoff, u_int8_t srelen, const u_char *bp, u_int len)
 			return;
 
 		memcpy(&a, bp, sizeof(a));
-		printf(" %s%s",
+		ND_PRINT((ndo, " %s%s",
 		    ((bp - up) == sreoff) ? "*" : "",
-		    inet_ntoa(a));
+		    inet_ntoa(a)));
 
 		bp += 4;
 		len -= 4;
@@ -370,21 +362,22 @@ gre_sre_ip_print(u_int8_t sreoff, u_int8_t srelen, const u_char *bp, u_int len)
 	}
 }
 
-void
-gre_sre_asn_print(u_int8_t sreoff, u_int8_t srelen, const u_char *bp, u_int len)
+static void
+gre_sre_asn_print(netdissect_options *ndo, uint8_t sreoff, uint8_t srelen,
+                  const u_char *bp, u_int len)
 {
 	const u_char *up = bp;
 
 	if (sreoff & 1) {
-		printf(", badoffset=%u", sreoff);
+		ND_PRINT((ndo, ", badoffset=%u", sreoff));
 		return;
 	}
 	if (srelen & 1) {
-		printf(", badlength=%u", srelen);
+		ND_PRINT((ndo, ", badlength=%u", srelen));
 		return;
 	}
 	if (sreoff >= srelen) {
-		printf(", badoff/len=%u/%u", sreoff, srelen);
+		ND_PRINT((ndo, ", badoff/len=%u/%u", sreoff, srelen));
 		return;
 	}
 
@@ -392,9 +385,9 @@ gre_sre_asn_print(u_int8_t sreoff, u_int8_t srelen, const u_char *bp, u_int len)
 		if (len < 2 || srelen == 0)
 			return;
 
-		printf(" %s%x",
+		ND_PRINT((ndo, " %s%x",
 		    ((bp - up) == sreoff) ? "*" : "",
-		    EXTRACT_16BITS(bp));
+		    EXTRACT_16BITS(bp)));
 
 		bp += 2;
 		len -= 2;
