@@ -436,12 +436,10 @@ again:
 		}
 		break;
 
-#ifdef INET6
 	case IPPROTO_IPV6:
 		/* ip6-in-ip encapsulation */
 		ip6_print(ndo, ipds->cp, ipds->len);
 		break;
-#endif /*INET6*/
 
 	case IPPROTO_RSVP:
 		rsvp_print(ndo, ipds->cp, ipds->len);
@@ -529,18 +527,17 @@ ip_print(netdissect_options *ndo,
 	struct protoent *proto;
 
 	ipds->ip = (const struct ip *)bp;
+	ND_TCHECK(ipds->ip->ip_vhl);
 	if (IP_V(ipds->ip) != 4) { /* print version if != 4 */
-	    ND_PRINT((ndo, "IP%u ", IP_V(ipds->ip)));
 	    if (IP_V(ipds->ip) == 6)
-	      ND_PRINT((ndo, ", wrong link-layer encapsulation"));
+	      ND_PRINT((ndo, "IP6, wrong link-layer encapsulation "));
+	    else
+	      ND_PRINT((ndo, "IP%u ", IP_V(ipds->ip)));
 	}
 	else if (!ndo->ndo_eflag)
 		ND_PRINT((ndo, "IP "));
 
-	if ((u_char *)(ipds->ip + 1) > ndo->ndo_snapend) {
-		ND_PRINT((ndo, "%s", tstr));
-		return;
-	}
+	ND_TCHECK(*ipds->ip);
 	if (length < sizeof (struct ip)) {
 		ND_PRINT((ndo, "truncated-ip %u", length));
 		return;
@@ -669,6 +666,11 @@ ip_print(netdissect_options *ndo,
 				ND_PRINT((ndo, " ip-proto-%d", ipds->ip->ip_p));
 		}
 	}
+	return;
+
+trunc:
+	ND_PRINT((ndo, "%s", tstr));
+	return;
 }
 
 void
@@ -685,11 +687,9 @@ ipN_print(netdissect_options *ndo, register const u_char *bp, register u_int len
 	case 4:
 		ip_print (ndo, bp, length);
 		return;
-#ifdef INET6
 	case 6:
 		ip6_print (ndo, bp, length);
 		return;
-#endif
 	default:
 		ND_PRINT((ndo, "unknown ip %d", IP_V(&hdr)));
 		return;
