@@ -27,11 +27,7 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-lwres.c,v 1.13 2004-03-24 01:54:29 guy Exp $ (LBL)";
-#endif
-
+#define NETDISSECT_REWORKED
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -48,9 +44,9 @@ static const char rcsid[] _U_ =
 #include "extract.h"                    /* must come after interface.h */
 
 /* BIND9 lib/lwres/include/lwres */
-typedef u_int32_t lwres_uint32_t;
-typedef u_int16_t lwres_uint16_t;
-typedef u_int8_t lwres_uint8_t;
+typedef uint32_t lwres_uint32_t;
+typedef uint16_t lwres_uint16_t;
+typedef uint8_t lwres_uint8_t;
 
 struct lwres_lwpacket {
 	lwres_uint32_t		length;
@@ -187,25 +183,21 @@ static const struct tok opcode[] = {
 extern const struct tok ns_type2str[];
 extern const struct tok ns_class2str[];
 
-static int lwres_printname(size_t, const char *);
-static int lwres_printnamelen(const char *);
-static int lwres_printbinlen(const char *);
-static int lwres_printaddr(lwres_addr_t *);
-
 static int
-lwres_printname(size_t l, const char *p0)
+lwres_printname(netdissect_options *ndo,
+                size_t l, const char *p0)
 {
 	const char *p;
 	size_t i;
 
 	p = p0;
 	/* + 1 for terminating \0 */
-	if (p + l + 1 > (const char *)snapend)
+	if (p + l + 1 > (const char *)ndo->ndo_snapend)
 		goto trunc;
 
-	printf(" ");
+	ND_PRINT((ndo, " "));
 	for (i = 0; i < l; i++)
-		safeputchar(*p++);
+		safeputchar(ndo, *p++);
 	p++;	/* skip terminating \0 */
 
 	return p - p0;
@@ -215,15 +207,16 @@ lwres_printname(size_t l, const char *p0)
 }
 
 static int
-lwres_printnamelen(const char *p)
+lwres_printnamelen(netdissect_options *ndo,
+                   const char *p)
 {
-	u_int16_t l;
+	uint16_t l;
 	int advance;
 
-	if (p + 2 > (const char *)snapend)
+	if (p + 2 > (const char *)ndo->ndo_snapend)
 		goto trunc;
 	l = EXTRACT_16BITS(p);
-	advance = lwres_printname(l, p + 2);
+	advance = lwres_printname(ndo, l, p + 2);
 	if (advance < 0)
 		goto trunc;
 	return 2 + advance;
@@ -233,21 +226,22 @@ lwres_printnamelen(const char *p)
 }
 
 static int
-lwres_printbinlen(const char *p0)
+lwres_printbinlen(netdissect_options *ndo,
+                  const char *p0)
 {
 	const char *p;
-	u_int16_t l;
+	uint16_t l;
 	int i;
 
 	p = p0;
-	if (p + 2 > (const char *)snapend)
+	if (p + 2 > (const char *)ndo->ndo_snapend)
 		goto trunc;
 	l = EXTRACT_16BITS(p);
-	if (p + 2 + l > (const char *)snapend)
+	if (p + 2 + l > (const char *)ndo->ndo_snapend)
 		goto trunc;
 	p += 2;
 	for (i = 0; i < l; i++)
-		printf("%02x", *p++);
+		ND_PRINT((ndo, "%02x", *p++));
 	return p - p0;
 
   trunc:
@@ -255,37 +249,38 @@ lwres_printbinlen(const char *p0)
 }
 
 static int
-lwres_printaddr(lwres_addr_t *ap)
+lwres_printaddr(netdissect_options *ndo,
+                lwres_addr_t *ap)
 {
-	u_int16_t l;
+	uint16_t l;
 	const char *p;
 	int i;
 
-	TCHECK(ap->length);
+	ND_TCHECK(ap->length);
 	l = EXTRACT_16BITS(&ap->length);
 	/* XXX ap points to packed struct */
 	p = (const char *)&ap->length + sizeof(ap->length);
-	TCHECK2(*p, l);
+	ND_TCHECK2(*p, l);
 
 	switch (EXTRACT_32BITS(&ap->family)) {
 	case 1:	/* IPv4 */
 		if (l < 4)
 			return -1;
-		printf(" %s", ipaddr_string(p));
+		ND_PRINT((ndo, " %s", ipaddr_string(ndo, p)));
 		p += sizeof(struct in_addr);
 		break;
 #ifdef INET6
 	case 2:	/* IPv6 */
 		if (l < 16)
 			return -1;
-		printf(" %s", ip6addr_string(p));
+		ND_PRINT((ndo, " %s", ip6addr_string(ndo, p)));
 		p += sizeof(struct in6_addr);
 		break;
 #endif
 	default:
-		printf(" %u/", EXTRACT_32BITS(&ap->family));
+		ND_PRINT((ndo, " %u/", EXTRACT_32BITS(&ap->family)));
 		for (i = 0; i < l; i++)
-			printf("%02x", *p++);
+			ND_PRINT((ndo, "%02x", *p++));
 	}
 
 	return p - (const char *)ap;
@@ -295,22 +290,23 @@ lwres_printaddr(lwres_addr_t *ap)
 }
 
 void
-lwres_print(register const u_char *bp, u_int length)
+lwres_print(netdissect_options *ndo,
+            register const u_char *bp, u_int length)
 {
 	const struct lwres_lwpacket *np;
-	u_int32_t v;
+	uint32_t v;
 	const char *s;
 	int response;
 	int advance;
 	int unsupported = 0;
 
 	np = (const struct lwres_lwpacket *)bp;
-	TCHECK(np->authlength);
+	ND_TCHECK(np->authlength);
 
-	printf(" lwres");
+	ND_PRINT((ndo, " lwres"));
 	v = EXTRACT_16BITS(&np->version);
-	if (vflag || v != LWRES_LWPACKETVERSION_0)
-		printf(" v%u", v);
+	if (ndo->ndo_vflag || v != LWRES_LWPACKETVERSION_0)
+		ND_PRINT((ndo, " v%u", v));
 	if (v != LWRES_LWPACKETVERSION_0) {
 		s = (const char *)np + EXTRACT_32BITS(&np->length);
 		goto tail;
@@ -321,25 +317,25 @@ lwres_print(register const u_char *bp, u_int length)
 	/* opcode and pktflags */
 	v = EXTRACT_32BITS(&np->opcode);
 	s = tok2str(opcode, "#0x%x", v);
-	printf(" %s%s", s, response ? "" : "?");
+	ND_PRINT((ndo, " %s%s", s, response ? "" : "?"));
 
 	/* pktflags */
 	v = EXTRACT_16BITS(&np->pktflags);
 	if (v & ~LWRES_LWPACKETFLAG_RESPONSE)
-		printf("[0x%x]", v);
+		ND_PRINT((ndo, "[0x%x]", v));
 
-	if (vflag > 1) {
-		printf(" (");	/*)*/
-		printf("serial:0x%x", EXTRACT_32BITS(&np->serial));
-		printf(" result:0x%x", EXTRACT_32BITS(&np->result));
-		printf(" recvlen:%u", EXTRACT_32BITS(&np->recvlength));
+	if (ndo->ndo_vflag > 1) {
+		ND_PRINT((ndo, " ("));	/*)*/
+		ND_PRINT((ndo, "serial:0x%x", EXTRACT_32BITS(&np->serial)));
+		ND_PRINT((ndo, " result:0x%x", EXTRACT_32BITS(&np->result)));
+		ND_PRINT((ndo, " recvlen:%u", EXTRACT_32BITS(&np->recvlength)));
 		/* BIND910: not used */
-		if (vflag > 2) {
-			printf(" authtype:0x%x", EXTRACT_16BITS(&np->authtype));
-			printf(" authlen:%u", EXTRACT_16BITS(&np->authlength));
+		if (ndo->ndo_vflag > 2) {
+			ND_PRINT((ndo, " authtype:0x%x", EXTRACT_16BITS(&np->authtype)));
+			ND_PRINT((ndo, " authlen:%u", EXTRACT_16BITS(&np->authlength)));
 		}
 		/*(*/
-		printf(")");
+		ND_PRINT((ndo, ")"));
 	}
 
 	/* per-opcode content */
@@ -350,7 +346,7 @@ lwres_print(register const u_char *bp, u_int length)
 		lwres_gabnrequest_t *gabn;
 		lwres_gnbarequest_t *gnba;
 		lwres_grbnrequest_t *grbn;
-		u_int32_t l;
+		uint32_t l;
 
 		gabn = NULL;
 		gnba = NULL;
@@ -361,51 +357,51 @@ lwres_print(register const u_char *bp, u_int length)
 			break;
 		case LWRES_OPCODE_GETADDRSBYNAME:
 			gabn = (lwres_gabnrequest_t *)(np + 1);
-			TCHECK(gabn->namelen);
+			ND_TCHECK(gabn->namelen);
 			/* XXX gabn points to packed struct */
 			s = (const char *)&gabn->namelen +
 			    sizeof(gabn->namelen);
 			l = EXTRACT_16BITS(&gabn->namelen);
 
 			/* BIND910: not used */
-			if (vflag > 2) {
-				printf(" flags:0x%x",
-				    EXTRACT_32BITS(&gabn->flags));
+			if (ndo->ndo_vflag > 2) {
+				ND_PRINT((ndo, " flags:0x%x",
+				    EXTRACT_32BITS(&gabn->flags)));
 			}
 
 			v = EXTRACT_32BITS(&gabn->addrtypes);
 			switch (v & (LWRES_ADDRTYPE_V4 | LWRES_ADDRTYPE_V6)) {
 			case LWRES_ADDRTYPE_V4:
-				printf(" IPv4");
+				ND_PRINT((ndo, " IPv4"));
 				break;
 			case LWRES_ADDRTYPE_V6:
-				printf(" IPv6");
+				ND_PRINT((ndo, " IPv6"));
 				break;
 			case LWRES_ADDRTYPE_V4 | LWRES_ADDRTYPE_V6:
-				printf(" IPv4/6");
+				ND_PRINT((ndo, " IPv4/6"));
 				break;
 			}
 			if (v & ~(LWRES_ADDRTYPE_V4 | LWRES_ADDRTYPE_V6))
-				printf("[0x%x]", v);
+				ND_PRINT((ndo, "[0x%x]", v));
 
-			advance = lwres_printname(l, s);
+			advance = lwres_printname(ndo, l, s);
 			if (advance < 0)
 				goto trunc;
 			s += advance;
 			break;
 		case LWRES_OPCODE_GETNAMEBYADDR:
 			gnba = (lwres_gnbarequest_t *)(np + 1);
-			TCHECK(gnba->addr);
+			ND_TCHECK(gnba->addr);
 
 			/* BIND910: not used */
-			if (vflag > 2) {
-				printf(" flags:0x%x",
-				    EXTRACT_32BITS(&gnba->flags));
+			if (ndo->ndo_vflag > 2) {
+				ND_PRINT((ndo, " flags:0x%x",
+				    EXTRACT_32BITS(&gnba->flags)));
 			}
 
 			s = (const char *)&gnba->addr;
 
-			advance = lwres_printaddr(&gnba->addr);
+			advance = lwres_printaddr(ndo, &gnba->addr);
 			if (advance < 0)
 				goto trunc;
 			s += advance;
@@ -413,19 +409,19 @@ lwres_print(register const u_char *bp, u_int length)
 		case LWRES_OPCODE_GETRDATABYNAME:
 			/* XXX no trace, not tested */
 			grbn = (lwres_grbnrequest_t *)(np + 1);
-			TCHECK(grbn->namelen);
+			ND_TCHECK(grbn->namelen);
 
 			/* BIND910: not used */
-			if (vflag > 2) {
-				printf(" flags:0x%x",
-				    EXTRACT_32BITS(&grbn->flags));
+			if (ndo->ndo_vflag > 2) {
+				ND_PRINT((ndo, " flags:0x%x",
+				    EXTRACT_32BITS(&grbn->flags)));
 			}
 
-			printf(" %s", tok2str(ns_type2str, "Type%d",
-			    EXTRACT_16BITS(&grbn->rdtype)));
+			ND_PRINT((ndo, " %s", tok2str(ns_type2str, "Type%d",
+			    EXTRACT_16BITS(&grbn->rdtype))));
 			if (EXTRACT_16BITS(&grbn->rdclass) != C_IN) {
-				printf(" %s", tok2str(ns_class2str, "Class%d",
-				    EXTRACT_16BITS(&grbn->rdclass)));
+				ND_PRINT((ndo, " %s", tok2str(ns_class2str, "Class%d",
+				    EXTRACT_16BITS(&grbn->rdclass))));
 			}
 
 			/* XXX grbn points to packed struct */
@@ -433,7 +429,7 @@ lwres_print(register const u_char *bp, u_int length)
 			    sizeof(grbn->namelen);
 			l = EXTRACT_16BITS(&grbn->namelen);
 
-			advance = lwres_printname(l, s);
+			advance = lwres_printname(ndo, l, s);
 			if (advance < 0)
 				goto trunc;
 			s += advance;
@@ -449,8 +445,8 @@ lwres_print(register const u_char *bp, u_int length)
 		lwres_gabnresponse_t *gabn;
 		lwres_gnbaresponse_t *gnba;
 		lwres_grbnresponse_t *grbn;
-		u_int32_t l, na;
-		u_int32_t i;
+		uint32_t l, na;
+		uint32_t i;
 
 		gabn = NULL;
 		gnba = NULL;
@@ -461,22 +457,22 @@ lwres_print(register const u_char *bp, u_int length)
 			break;
 		case LWRES_OPCODE_GETADDRSBYNAME:
 			gabn = (lwres_gabnresponse_t *)(np + 1);
-			TCHECK(gabn->realnamelen);
+			ND_TCHECK(gabn->realnamelen);
 			/* XXX gabn points to packed struct */
 			s = (const char *)&gabn->realnamelen +
 			    sizeof(gabn->realnamelen);
 			l = EXTRACT_16BITS(&gabn->realnamelen);
 
 			/* BIND910: not used */
-			if (vflag > 2) {
-				printf(" flags:0x%x",
-				    EXTRACT_32BITS(&gabn->flags));
+			if (ndo->ndo_vflag > 2) {
+				ND_PRINT((ndo, " flags:0x%x",
+				    EXTRACT_32BITS(&gabn->flags)));
 			}
 
-			printf(" %u/%u", EXTRACT_16BITS(&gabn->naliases),
-			    EXTRACT_16BITS(&gabn->naddrs));
+			ND_PRINT((ndo, " %u/%u", EXTRACT_16BITS(&gabn->naliases),
+			    EXTRACT_16BITS(&gabn->naddrs)));
 
-			advance = lwres_printname(l, s);
+			advance = lwres_printname(ndo, l, s);
 			if (advance < 0)
 				goto trunc;
 			s += advance;
@@ -484,7 +480,7 @@ lwres_print(register const u_char *bp, u_int length)
 			/* aliases */
 			na = EXTRACT_16BITS(&gabn->naliases);
 			for (i = 0; i < na; i++) {
-				advance = lwres_printnamelen(s);
+				advance = lwres_printnamelen(ndo, s);
 				if (advance < 0)
 					goto trunc;
 				s += advance;
@@ -493,7 +489,7 @@ lwres_print(register const u_char *bp, u_int length)
 			/* addrs */
 			na = EXTRACT_16BITS(&gabn->naddrs);
 			for (i = 0; i < na; i++) {
-				advance = lwres_printaddr((lwres_addr_t *)s);
+				advance = lwres_printaddr(ndo, (lwres_addr_t *)s);
 				if (advance < 0)
 					goto trunc;
 				s += advance;
@@ -501,21 +497,21 @@ lwres_print(register const u_char *bp, u_int length)
 			break;
 		case LWRES_OPCODE_GETNAMEBYADDR:
 			gnba = (lwres_gnbaresponse_t *)(np + 1);
-			TCHECK(gnba->realnamelen);
+			ND_TCHECK(gnba->realnamelen);
 			/* XXX gnba points to packed struct */
 			s = (const char *)&gnba->realnamelen +
 			    sizeof(gnba->realnamelen);
 			l = EXTRACT_16BITS(&gnba->realnamelen);
 
 			/* BIND910: not used */
-			if (vflag > 2) {
-				printf(" flags:0x%x",
-				    EXTRACT_32BITS(&gnba->flags));
+			if (ndo->ndo_vflag > 2) {
+				ND_PRINT((ndo, " flags:0x%x",
+				    EXTRACT_32BITS(&gnba->flags)));
 			}
 
-			printf(" %u", EXTRACT_16BITS(&gnba->naliases));
+			ND_PRINT((ndo, " %u", EXTRACT_16BITS(&gnba->naliases)));
 
-			advance = lwres_printname(l, s);
+			advance = lwres_printname(ndo, l, s);
 			if (advance < 0)
 				goto trunc;
 			s += advance;
@@ -523,7 +519,7 @@ lwres_print(register const u_char *bp, u_int length)
 			/* aliases */
 			na = EXTRACT_16BITS(&gnba->naliases);
 			for (i = 0; i < na; i++) {
-				advance = lwres_printnamelen(s);
+				advance = lwres_printnamelen(ndo, s);
 				if (advance < 0)
 					goto trunc;
 				s += advance;
@@ -532,29 +528,29 @@ lwres_print(register const u_char *bp, u_int length)
 		case LWRES_OPCODE_GETRDATABYNAME:
 			/* XXX no trace, not tested */
 			grbn = (lwres_grbnresponse_t *)(np + 1);
-			TCHECK(grbn->nsigs);
+			ND_TCHECK(grbn->nsigs);
 
 			/* BIND910: not used */
-			if (vflag > 2) {
-				printf(" flags:0x%x",
-				    EXTRACT_32BITS(&grbn->flags));
+			if (ndo->ndo_vflag > 2) {
+				ND_PRINT((ndo, " flags:0x%x",
+				    EXTRACT_32BITS(&grbn->flags)));
 			}
 
-			printf(" %s", tok2str(ns_type2str, "Type%d",
-			    EXTRACT_16BITS(&grbn->rdtype)));
+			ND_PRINT((ndo, " %s", tok2str(ns_type2str, "Type%d",
+			    EXTRACT_16BITS(&grbn->rdtype))));
 			if (EXTRACT_16BITS(&grbn->rdclass) != C_IN) {
-				printf(" %s", tok2str(ns_class2str, "Class%d",
-				    EXTRACT_16BITS(&grbn->rdclass)));
+				ND_PRINT((ndo, " %s", tok2str(ns_class2str, "Class%d",
+				    EXTRACT_16BITS(&grbn->rdclass))));
 			}
-			printf(" TTL ");
-			relts_print(EXTRACT_32BITS(&grbn->ttl));
-			printf(" %u/%u", EXTRACT_16BITS(&grbn->nrdatas),
-			    EXTRACT_16BITS(&grbn->nsigs));
+			ND_PRINT((ndo, " TTL "));
+			relts_print(ndo, EXTRACT_32BITS(&grbn->ttl));
+			ND_PRINT((ndo, " %u/%u", EXTRACT_16BITS(&grbn->nrdatas),
+			    EXTRACT_16BITS(&grbn->nsigs)));
 
 			/* XXX grbn points to packed struct */
 			s = (const char *)&grbn->nsigs+ sizeof(grbn->nsigs);
 
-			advance = lwres_printnamelen(s);
+			advance = lwres_printnamelen(ndo, s);
 			if (advance < 0)
 				goto trunc;
 			s += advance;
@@ -563,7 +559,7 @@ lwres_print(register const u_char *bp, u_int length)
 			na = EXTRACT_16BITS(&grbn->nrdatas);
 			for (i = 0; i < na; i++) {
 				/* XXX should decode resource data */
-				advance = lwres_printbinlen(s);
+				advance = lwres_printbinlen(ndo, s);
 				if (advance < 0)
 					goto trunc;
 				s += advance;
@@ -573,7 +569,7 @@ lwres_print(register const u_char *bp, u_int length)
 			na = EXTRACT_16BITS(&grbn->nsigs);
 			for (i = 0; i < na; i++) {
 				/* XXX how should we print it? */
-				advance = lwres_printbinlen(s);
+				advance = lwres_printbinlen(ndo, s);
 				if (advance < 0)
 					goto trunc;
 				s += advance;
@@ -588,14 +584,13 @@ lwres_print(register const u_char *bp, u_int length)
   tail:
 	/* length mismatch */
 	if (EXTRACT_32BITS(&np->length) != length) {
-		printf(" [len: %u != %u]", EXTRACT_32BITS(&np->length),
-		    length);
+		ND_PRINT((ndo, " [len: %u != %u]", EXTRACT_32BITS(&np->length),
+		    length));
 	}
 	if (!unsupported && s < (const char *)np + EXTRACT_32BITS(&np->length))
-		printf("[extra]");
+		ND_PRINT((ndo, "[extra]"));
 	return;
 
   trunc:
-	printf("[|lwres]");
-	return;
+	ND_PRINT((ndo, "[|lwres]"));
 }
