@@ -15,20 +15,14 @@
  * LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  * FOR A PARTICULAR PURPOSE.
  */
-#ifndef lint
-static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-msdp.c,v 1.7 2005/04/06 21:32:41 mcr Exp $";
-#endif
 
+#define NETDISSECT_REWORKED
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include <tcpdump-stdinc.h>
 
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "interface.h"
 #include "addrtoname.h"
 #include "extract.h"
@@ -36,23 +30,23 @@ static const char rcsid[] _U_ =
 #define MSDP_TYPE_MAX	7
 
 void
-msdp_print(const unsigned char *sp, u_int length)
+msdp_print(netdissect_options *ndo, const u_char *sp, u_int length)
 {
 	unsigned int type, len;
 
-	TCHECK2(*sp, 3);
+	ND_TCHECK2(*sp, 3);
 	/* See if we think we're at the beginning of a compound packet */
 	type = *sp;
 	len = EXTRACT_16BITS(sp + 1);
 	if (len > 1500 || len < 3 || type == 0 || type > MSDP_TYPE_MAX)
 		goto trunc;	/* not really truncated, but still not decodable */
-	(void)printf(" msdp:");
+	ND_PRINT((ndo, " msdp:"));
 	while (length > 0) {
-		TCHECK2(*sp, 3);
+		ND_TCHECK2(*sp, 3);
 		type = *sp;
 		len = EXTRACT_16BITS(sp + 1);
-		if (len > 1400 || vflag)
-			printf(" [len %u]", len);
+		if (len > 1400 || ndo->ndo_vflag)
+			ND_PRINT((ndo, " [len %u]", len));
 		if (len < 3)
 			goto trunc;
 		sp += 3;
@@ -61,35 +55,35 @@ msdp_print(const unsigned char *sp, u_int length)
 		case 1:	/* IPv4 Source-Active */
 		case 3: /* IPv4 Source-Active Response */
 			if (type == 1)
-				(void)printf(" SA");
+				ND_PRINT((ndo, " SA"));
 			else
-				(void)printf(" SA-Response");
-			TCHECK(*sp);
-			(void)printf(" %u entries", *sp);
+				ND_PRINT((ndo, " SA-Response"));
+			ND_TCHECK(*sp);
+			ND_PRINT((ndo, " %u entries", *sp));
 			if ((u_int)((*sp * 12) + 8) < len) {
-				(void)printf(" [w/data]");
-				if (vflag > 1) {
-					(void)printf(" ");
-					ip_print(gndo, sp + *sp * 12 + 8 - 3,
+				ND_PRINT((ndo, " [w/data]"));
+				if (ndo->ndo_vflag > 1) {
+					ND_PRINT((ndo, " "));
+					ip_print(ndo, sp + *sp * 12 + 8 - 3,
 					         len - (*sp * 12 + 8));
 				}
 			}
 			break;
 		case 2:
-			(void)printf(" SA-Request");
-			TCHECK2(*sp, 5);
-			(void)printf(" for %s", ipaddr_string(sp + 1));
+			ND_PRINT((ndo, " SA-Request"));
+			ND_TCHECK2(*sp, 5);
+			ND_PRINT((ndo, " for %s", ipaddr_string(ndo, sp + 1)));
 			break;
 		case 4:
-			(void)printf(" Keepalive");
+			ND_PRINT((ndo, " Keepalive"));
 			if (len != 3)
-				(void)printf("[len=%d] ", len);
+				ND_PRINT((ndo, "[len=%d] ", len));
 			break;
 		case 5:
-			(void)printf(" Notification");
+			ND_PRINT((ndo, " Notification"));
 			break;
 		default:
-			(void)printf(" [type=%d len=%d]", type, len);
+			ND_PRINT((ndo, " [type=%d len=%d]", type, len));
 			break;
 		}
 		sp += (len - 3);
@@ -97,7 +91,7 @@ msdp_print(const unsigned char *sp, u_int length)
 	}
 	return;
 trunc:
-	(void)printf(" [|msdp]");
+	ND_PRINT((ndo, " [|msdp]"));
 }
 
 /*
