@@ -145,11 +145,11 @@ nflog_if_print(netdissect_options *ndo,
 		ip_print(ndo, p, length);
 		break;
 
-#ifdef INET6
+#ifdef AF_INET6
 	case AF_INET6:
 		ip6_print(ndo, p, length);
 		break;
-#endif /*INET6*/
+#endif /* AF_INET6 */
 
 	default:
 		if (!ndo->ndo_eflag)
