@@ -19,22 +19,13 @@
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#ifndef lint
-static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-raw.c,v 1.41 2003-11-16 09:36:34 guy Exp $ (LBL)";
-#endif
-
+#define NETDISSECT_REWORKED
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include <tcpdump-stdinc.h>
 
-#include <pcap.h>
-#include <stdio.h>
-#include <string.h>
-
-#include "addrtoname.h"
 #include "interface.h"
 
 /*
@@ -42,12 +33,12 @@ static const char rcsid[] _U_ =
  */
 
 u_int
-raw_if_print(const struct pcap_pkthdr *h, const u_char *p)
+raw_if_print(netdissect_options *ndo, const struct pcap_pkthdr *h, const u_char *p)
 {
-	if (eflag)
-		printf("ip: ");
+	if (ndo->ndo_eflag)
+		ND_PRINT((ndo, "ip: "));
 
-	ipN_print(p, h->len);
+	ipN_print(ndo, p, h->len);
 
 	return (0);
 }
