@@ -19,11 +19,7 @@
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#ifndef lint
-static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/tcpdump/print-rt6.c,v 1.27 2005-04-20 22:34:57 guy Exp $";
-#endif
-
+#define NETDISSECT_REWORKED
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -32,7 +28,7 @@ static const char rcsid[] _U_ =
 
 #include <tcpdump-stdinc.h>
 
-#include <stdio.h>
+#include <string.h>
 
 #include "ip6.h"
 
@@ -41,25 +37,26 @@ static const char rcsid[] _U_ =
 #include "extract.h"
 
 int
-rt6_print(register const u_char *bp, const u_char *bp2 _U_)
+rt6_print(netdissect_options *ndo, register const u_char *bp, const u_char *bp2 _U_)
 {
 	register const struct ip6_rthdr *dp;
 	register const struct ip6_rthdr0 *dp0;
 	register const u_char *ep;
 	int i, len;
 	register const struct in6_addr *addr;
+	const struct in6_addr *last_addr = NULL;
 
 	dp = (struct ip6_rthdr *)bp;
 	len = dp->ip6r_len;
 
 	/* 'ep' points to the end of available data. */
-	ep = snapend;
+	ep = ndo->ndo_snapend;
 
-	TCHECK(dp->ip6r_segleft);
+	ND_TCHECK(dp->ip6r_segleft);
 
-	printf("srcrt (len=%d", dp->ip6r_len);	/*)*/
-	printf(", type=%d", dp->ip6r_type);
-	printf(", segleft=%d", dp->ip6r_segleft);
+	ND_PRINT((ndo, "srcrt (len=%d", dp->ip6r_len));	/*)*/
+	ND_PRINT((ndo, ", type=%d", dp->ip6r_type));
+	ND_PRINT((ndo, ", segleft=%d", dp->ip6r_segleft));
 
 	switch (dp->ip6r_type) {
 #ifndef IPV6_RTHDR_TYPE_0
@@ -72,10 +69,10 @@ rt6_print(register const u_char *bp, const u_char *bp2 _U_)
 	case IPV6_RTHDR_TYPE_2:			/* Mobile IPv6 ID-20 */
 		dp0 = (struct ip6_rthdr0 *)dp;
 
-		TCHECK(dp0->ip6r0_reserved);
-		if (dp0->ip6r0_reserved || vflag) {
-			printf(", rsv=0x%0x",
-			    EXTRACT_32BITS(&dp0->ip6r0_reserved));
+		ND_TCHECK(dp0->ip6r0_reserved);
+		if (dp0->ip6r0_reserved || ndo->ndo_vflag) {
+			ND_PRINT((ndo, ", rsv=0x%0x",
+			    EXTRACT_32BITS(&dp0->ip6r0_reserved)));
 		}
 
 		if (len % 2 == 1)
@@ -86,11 +83,20 @@ rt6_print(register const u_char *bp, const u_char *bp2 _U_)
 			if ((u_char *)(addr + 1) > ep)
 				goto trunc;
 
-			printf(", [%d]%s", i, ip6addr_string(addr));
+			ND_PRINT((ndo, ", [%d]%s", i, ip6addr_string(ndo, addr)));
+			last_addr = addr;
 			addr++;
 		}
+		/*
+		 * the destination address used in the pseudo-header is that of the final
+		 * destination : the last address of the routing header
+		 */
+		if (last_addr != NULL) {
+			struct ip6_hdr *ip6 = (struct ip6_hdr *)bp2;
+			UNALIGNED_MEMCPY(&ip6->ip6_dst, last_addr, sizeof (struct in6_addr));
+		}
 		/*(*/
-		printf(") ");
+		ND_PRINT((ndo, ") "));
 		return((dp0->ip6r0_len + 1) << 3);
 		break;
 	default:
@@ -99,7 +105,7 @@ rt6_print(register const u_char *bp, const u_char *bp2 _U_)
 	}
 
  trunc:
-	fputs("[|srcrt]", stdout);
+	ND_PRINT((ndo, "[|srcrt]"));
 	return -1;
 }
 #endif /* INET6 */
