@@ -62,7 +62,7 @@ sl_if_print(netdissect_options *ndo,
 	register u_int length = h->len;
 	register const struct ip *ip;
 
-	if (caplen < SLIP_HDRLEN) {
+	if (caplen < SLIP_HDRLEN || length < SLIP_HDRLEN) {
 		ND_PRINT((ndo, "%s", tstr));
 		return (caplen);
 	}
@@ -78,11 +78,9 @@ sl_if_print(netdissect_options *ndo,
 	case 4:
 	        ip_print(ndo, (u_char *)ip, length);
 		break;
-#ifdef INET6
 	case 6:
 		ip6_print(ndo, (u_char *)ip, length);
 		break;
-#endif
 	default:
 		ND_PRINT((ndo, "ip v%d", IP_V(ip)));
 	}
